/**
 * 
 */
package com.platomix.userprofile.handler.support;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import com.alibaba.fastjson.JSONObject;
import com.platomix.userprofile.enums.DsEnum;
import com.platomix.userprofile.model.ShellCommond;
import com.platomix.userprofile.service.DynamicExecuteService;
import com.platomix.userprofile.service.MobileRangeService;
import com.platomix.userprofile.utils.ShellUtils;
import com.platomix.userprofile.utils.StrBuilder;
import com.platomix.userprofile.utils.TagKit;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author felix
 * 信息化数据处理 帮助类
 */
@Slf4j
@Component
public class XxhTagManager {

	final static String ETL_TABLE_NAME = "ads.XXH_ALL_MONTH_ETL2";
	@Autowired
	protected DynamicExecuteService dynamicExecuteService;

	@Autowired
	private MobileRangeService mobileRangeService;

	@Value("${spring.datasource.dynamic.datasource.clickhouse.username}")
	private String clickhouseUserName;
	@Value("${spring.datasource.dynamic.datasource.clickhouse.password}")
	private String clickhousePassWord;

	@Value("${spring.datasource.dynamic.datasource.clickhouse91.username}")
	private String clickhouseUserName91;
	@Value("${spring.datasource.dynamic.datasource.clickhouse91.password}")
	private String clickhousePassWord91;

	final static long _STEP=17000000L;
	/**
	 * 从etl表向 xxh表转化 转化时 重复消除重复
	 * @param month 月份
	 */
	private void xxhFromEtl(String month, String mobileLeft, String mobileRight) {
		String lrMobileSql = mobileLeft == null || mobileRight == null ? "" : String.format(" and DEVICE_NUMBER>='%s' and DEVICE_NUMBER<'%s'", mobileLeft, mobileRight);
		StrBuilder sqlBuilder = new StrBuilder();
		sqlBuilder.appendLn("insert into ads.XXH_ALL_MONTH")
			    .appendLn("select distinct MONTH_ID, SEARCH_FLAG, DEVICE_NUMBER, PROV_DESC, UPPER(SERVICE_TYPE) as SERVICE_TYPE, toUInt32OrZero(INNET_MONTHS) INNET_MONTHS, CUST_SEX, CERT_AGE,")
				.appendLn("TOTAL_FLUX, TOTAL_FEE, ifNull(MON_TOTAL_FEE,'0') AS MON_TOTAL_FEE, PRODUCT_CLASS_DESC, null as MON_FEE, FLUX_M, RESOURCE_NAME,")
				.appendLn("PKG_NAME, MANU_NAME, MODEL_NAME, IS_ACCT ,TOTAL_FLUX_VIDEO, AREA as ARAE")
				.appendLn(String.format("from %s",ETL_TABLE_NAME))
				.appendLn("where MONTH_ID ='"+month+"'" + lrMobileSql)
				.appendLn(String.format("and DEVICE_NUMBER not in (SELECT DEVICE_NUMBER from ads.XXH_ALL_MONTH where MONTH_ID ='%s' and SEARCH_FLAG in('b','B')  %s)",month,lrMobileSql));
		log.info("\n" + sqlBuilder.toString());
		dynamicExecuteService.execute(DsEnum.clickhouse, sqlBuilder.toString(), "ETL导入数据转化为XXH基础数据1");

		StrBuilder sqlBuilderApp = new StrBuilder();
		sqlBuilderApp.appendLn("insert into ads.XXH_ALL_APP_MONTH")
				.appendLn("select MONTH_ID,DEVICE_NUMBER, APP_PRE, CON_PRE, APP_NAME, Max(toUInt32OrZero(DAYS)) as DAYS,")
				.appendLn("toUInt32OrZero(TIMES) as TIMES, toUInt32OrZero(MINS) as MINS, toFloat64OrZero(ONE_FLUX) as ONE_FLUX")
				.appendLn("from ")
				.appendLn("( select MONTH_ID,DEVICE_NUMBER, APP_PRE, CON_PRE, APP_NAME, DAYS,TIMES,MINS,ifNull(ONE_FLUX,'0') as  ONE_FLUX")
				.appendLn("from  "+ETL_TABLE_NAME+" where MONTH_ID ='"+month+"'" + lrMobileSql)
				.appendLn(String.format("and DEVICE_NUMBER not in (SELECT DEVICE_NUMBER from ads.XXH_ALL_MONTH where MONTH_ID ='%s' and SEARCH_FLAG in('b','B')  %s)",month,lrMobileSql))
				.appendLn(") s")
				.appendLn("group by MONTH_ID,DEVICE_NUMBER, APP_PRE, CON_PRE, APP_NAME, TIMES, MINS, ONE_FLUX");
		log.info("\n" + sqlBuilderApp.toString());
		dynamicExecuteService.execute(DsEnum.clickhouse, sqlBuilderApp.toString(), "ETL导入数据转化为XXH基础数据2");


	}
	
	/**
	 * 从etl表向 xxh表转化
	 * @param month 月份
	 */
	public void xxhFromEtlSp(String month) {
		SortedSet<String> mobileList = mobileRangeService.getList(MobileRangeService.DEFAULT_STEP);
		String mobileLeft = mobileList.first();
		for(Iterator<String> it = mobileList.iterator(); it.hasNext(); ) {
			String mobileRight = it.next();
			if(mobileLeft.equals(mobileRight)) {
				continue;
			}
			xxhFromEtl(month, mobileLeft, mobileRight);
			mobileLeft = mobileRight;
		}
	}

	/**
	 * 删除表！！！
	 */
	public void deleteTables(String listStr) {
		//		String listStr=
//				"table1\n" +
//				"table2\n" +
//				"table3";
	    String[] s1=listStr.split("\\n");
		Arrays.stream(s1).forEach(str-> {
//			if (str.indexOf("_clu") > 0 || str.indexOf("_ALL") > 0)
				log.info("\n" + String.format("drop table if exists ads.%s", str));
				dynamicExecuteService.execute(DsEnum.clickhouse, String.format("drop table if exists %s",str), "");
				try {
					TimeUnit.SECONDS.sleep(2);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}

		});
	}

	/**
	 * 像91创建需要导的表
	 * @param listStr 要处理的表名 用\n分割（直接从文本文件里复制所得）
	 */
	public void createTables(String listStr) throws IOException {
		String[] s1=listStr.split("\\n");
		FileUtils.forceMkdir(new File(ShellUtils.SHELL_DIC));
		Arrays.stream(s1).forEach(str-> {
//			if (str.indexOf("_clu") > 0 || str.indexOf("_ALL") > 0)
			JSONObject jsonObject =dynamicExecuteService.get(DsEnum.clickhouse, String.format("select create_table_query from `system`.tables t where name ='%s' ",str));
			if(jsonObject != null ){
				String runCreateSql=jsonObject.getString("create_table_query");
				log.info("\n" + runCreateSql);
				dynamicExecuteService.execute(DsEnum.clickhouse91,runCreateSql.replace("CREATE TABLE","CREATE TABLE if not exists"), "手动创建复制库表！");

				//写shell插入数据
				StrBuilder builder=new StrBuilder();
				builder.append("echo 'select * ")
						.append(String.format(" from ads.%s ' | ",str))
						.append(String.format(" curl %s:8123?database=ads -u%s:%s -d @- | ","10.0.1.135",clickhouseUserName,clickhousePassWord))
						.append(String.format(" docker run -i --rm yandex/clickhouse-client --host %s --port 9007 --user %s --password %s --input_format_allow_errors_num=100","10.0.1.91",clickhouseUserName91,clickhousePassWord91))
						.append(String.format(" --query='INSERT INTO ads.%s FORMAT TabSeparated' ",str));
				try {
					ShellUtils.shellToCluster(new ShellCommond(str+"_91",false,true,builder.toString()));
				} catch (IOException | InterruptedException e) {
					e.printStackTrace();
				}
			}
		});
	}
}
